package org.qbit.service.impl;

import org.boon.Boon;
import org.boon.Lists;
import org.boon.Str;
import org.boon.collections.MultiMap;
import org.boon.core.Conversions;
import org.boon.core.Sys;
import org.junit.Before;
import org.junit.Test;
import org.qbit.Factory;
import org.qbit.QBit;
import org.qbit.message.MethodCall;
import org.qbit.message.Response;
import org.qbit.queue.ReceiveQueue;
import org.qbit.service.ServiceBundle;

import java.util.List;

import static org.boon.Boon.puts;

public class ServiceBundleImplTest {


    ServiceBundle serviceBundle;
    ServiceBundleImpl serviceBundleImpl;
    AdderService adderService;

    Factory factory;
    MultiMap<String, String> params = null;
    MethodCall<Object> call = null;


    ReceiveQueue<Response<Object>> responseReceiveQueue = null;

    Response<Object> response;

    Object responseBody = null;




    public static class AdderService {
        int sum;
        public int add(int a, int b) {

            puts("ADDER SERVICE CALLED", a, b);
            sum += (a+b);
            return a+b;
        }
    }


    @Before
    public void before() {

        factory = QBit.factory();
        final ServiceBundle bundle = factory.createBundle("/foo");
        serviceBundle = bundle;
        serviceBundleImpl = (ServiceBundleImpl) bundle;
        adderService = new AdderService();
    }

    @Test
    public void testAddress() throws Exception {

        Str.equalsOrDie("/foo", serviceBundle.address());

    }

    @Test
    public void testAddService() throws Exception {

        serviceBundle.addService("/adder", adderService);
        final List<String> endPoints = serviceBundle.endPoints();
        puts(endPoints);
        endPoints.contains("/foo/adder");
    }

    @Test
    public void testResponses() throws Exception {

        call = factory.createMethodCallByAddress("/foo/adder/add", "", Lists.list(1, 2), params);
        serviceBundle.addService("/adder", adderService);

        serviceBundle.call(call);

        serviceBundle.flushSends();

        Sys.sleep(1000);

        responseReceiveQueue = serviceBundle.responses();

        response = responseReceiveQueue.pollWait();

        responseBody = response.body();

        int sum = Conversions.toInt(responseBody);

        Boon.equalsOrDie("Sum should be 3", 3, sum);

        serviceBundle.stop();
    }

    @Test
    public void testCall() throws Exception {

    }
}
